var __tag_end        = "<!--END-->";
var __hexstring = "0123456789ABCDEF";

function DEBUG( message ) {
    var elem = document.getElementById( 'status' );
    if ( elem ) elem.innerText = message;
}

function __convert_to_hex( num ) {
    var div = parseInt( num / 16 );
    var rem = num - div * 16;
    if ( div < 0 ) div = 0;
    if ( div > 15 ) div = 15;
    if ( rem < 0 ) rem = 0;
    if ( rem > 15 ) rem = 15;
    return __hexstring.charAt( div ) + __hexstring.charAt( rem );
}

function showStatus( message ) {
    var elem_status = document.getElementById( "status" );
    if ( elem_status ) {
        elem_status.innerText = message;
    }
}


function deleteCache() {
    if ( window.widget ) {
        widget.system( "/usr/bin/python remove_cache.py", null );
    }
}

function BlinkTimer( elem ) {
    if ( elem == undefined || elem == null ) {
        throw new Error( "No element specified" );
    }

    this.element  = elem;
    this.timer    = null;
    this.counter  = 0;
    this.interval = 13;
    this.isRunning = false;
    this.initialColor = elem.style.color;

    this.start   = _BlinkTimer_start;
    this.tick    = _BlinkTimer_tick;
    this.cancel  = _BlinkTimer_cancel;
}

function _BlinkTimer_start() {
    if ( this.isRunning ) {
        this.cancel();
    }
    var self = this;
    this.isRunning = true;
    if ( this.timer != null ) {
        clearInterval( this.timer );
        this.timer = null;
    }
    this.counter = 0;
    this.timer     = setInterval( function(){ self.tick(); }, this.interval );
}

function _BlinkTimer_tick() {
    if ( this.isRunning == false ) return;
    if ( this.element != null ) {
        //this.element.innerText = this.counter;
        var count = this.counter * 2;
        if ( count > 255 ) {
            count = 512 - this.counter * 2;
        }
        var r = __convert_to_hex( count );
        var g = r;
        var b = r;

        this.element.style.color = "#" + r + g + b;
        this.counter = ( this.counter + 4 ) % 256;
    }
}

function _BlinkTimer_cancel() {
    if ( this.timer != null ) {
        clearInterval( this.timer );
        this.timer = null;
    }
    this.isRunning = false;
    this.element.style.color = this.initialColor;
}


// JMA retrieving object
function JMALoader( tableOutputElement, infoOutputElement ) {
    if ( tableOutputElement == undefined || tableOutputElement == null
         || infoOutputElement == undefined || infoOutputElement == null ) {
        throw new Error( "table and information element must be given." );
    }
    this.tableElement = tableOutputElement;
    this.infoElement  = infoOutputElement;

    this.command      = null;
    this.python       = "/usr/bin/python get_jma_data.py";

    this.loadingURL   = null;
    this.previousURL  = null;
    this.baseURL      = "http://www.jma.go.jp/JMA_HP/jp/quake/";
    this.currentURL   = "http://www.jma.go.jp/JMA_HP/jp/quake/";
    this.posteriorURL = null;

    this.isRunning    = false;
    this.errorFlag    = false;
    this.endHandler   = null;

    this.getPreviousURL   = _JMALoader_getPreviousURL;
    this.getPosteriorURL  = _JMALoader_getPosteriorURL;
    this.getURL           = _JMALoader_getURL;
    this.getBaseURL       = _JMALoader_getBaseURL;
    this.succeeded        = _JMALoader_succeeded;

    this.processHTML      = _JMALoader_processHTML;
    this.load             = _JMALoader_load;
    this.forceLoading     = _JMALoader_forceLoading;

    this.cancel           = _JMALoader_cancel;

    this.setEndHandler    = _JMALoader_setEndHandler;
    this.removeEndHandler = _JMALoader_removeEndHandler;
}

function _JMALoader_succeeded() {
    return !( this.errorFlag );
}

function _JMALoader_setEndHandler( callBack ) {
    this.endHandler = callBack;
}

function _JMALoader_removeEndHandler() {
    this.endHandler = null;
}

function _JMALoader_getBaseURL() {
    return this.baseURL;
}

function _JMALoader_getPreviousURL() {
    return this.previousURL;
}

function _JMALoader_getPosteriorURL() {
    return this.posteriorURL;
}

function _JMALoader_getURL() {
    return this.currentURL;
}


function _JMALoader_processHTML( arg ) {
    if ( this.command == null ) return;
    var output  = this.command.outputString;
    if ( output == undefined || output == null || output == "" ) {
        var error = this.command.errorString;
        showStatus( error );
        return;
    }
    var outputs = output.split( "\n" );

    var num_info_start = 3;
    var comelem = document.createElement( "div" );
    var tabelem = document.createElement( "table" );
    tabelem.setAttribute( "id", "regiontable" );

    var header = document.createElement( "tr" );
    var header_titles = [ "県名", "震度", "地域" ];
    for ( var i = 0; i < 3; i++ ) {
        var th = document.createElement( "th" );
        th.appendChild( document.createTextNode( header_titles[ i ] ) );
        if ( i == 2 ) {
            th.setAttribute( 'class', 'area' );
        }
        header.appendChild( th );
    }
    tabelem.appendChild( header );

    if ( outputs.length >= num_info_start ) {
        this.currentURL = outputs[ 0 ].substring( 4, outputs[ 0 ].length );
        this.previousURL = outputs[ 1 ].substring( 4, outputs[ 1 ].length );
        this.posteriorURL = outputs[ 2 ].substring( 4, outputs[ 2 ].length );
        
        for ( var i = num_info_start; i < outputs.length; i++ ) {
            if ( outputs[ i ] == "//-" ) {
                for ( var k = i + 1; k < outputs.length; k++ ) {
                    var items = outputs[ k ].split( "," );
                    if ( items != null && items.length >= 3 ) {
                        var td;
                        var tr = document.createElement( "tr" );
                        for ( var j = 0; j < 3; j++ ) {
                            td = document.createElement( "td" );
                            td.appendChild( document.createTextNode( items[ j ] ) );
                            if ( j == 2 ) {
                                td.setAttribute( "class", "area" );
                            }
                            tr.appendChild( td );
                        }
                        tabelem.appendChild( tr );                        
                    }
                }
                this.tableElement.removeChild( document.getElementById( 'regiontable' ) );
                var children = this.tableElement.childNodes;
                for ( var j = 0; j < children.length; j++ ) {
                    this.tableElement.removeChild( children[ j ] );
                }

                this.tableElement.appendChild( tabelem );
                break;
            } else {
                comelem.appendChild( document.createTextNode( outputs[ i ] ) );
                comelem.appendChild( document.createElement( "br" ) );
            }
        }

        if ( this.infoElement != null ) {
            var nodes = this.infoElement.childNodes;
            for ( var i = nodes.length - 1; i >= 0; i-- ) {
                this.infoElement.removeChild( nodes[ i ] );
            }
            this.infoElement.appendChild( comelem );
        }

        this.errorFlag = false;
    } else {
         if ( output.indexOf( "NETWORK ERROR" ) >= 0 ) {
             showStatus( "ダウンロードに失敗しました" );
         } else {
             showStatus( output );
         }
        this.errorFlag = true;
    }

    // notify finished
    if ( this.endHandler != null ) {
        this.endHandler( this );
    }
}

function _JMALoader_load( the_url, forceFlag ) {
    if ( this.isRunning ) {
        this.cancel();
    }
    
    this.isRunning = true;
    if ( forceFlag == undefined || forceFlag == null ) {
        forceFlag = false;
    }
    if ( the_url == undefined || the_url == null ) {
        throw new Error( "No url specified" );
    }

    this.url_loading = the_url;
    if ( window.widget ) {
        if ( the_url == "" ) {
            this.pycommand = this.python;
        } else {
            this.pycommand = this.python + " \"" + the_url + "\"";
        }
        if ( forceFlag ) {
            this.pycommand += " -force";
        }
        //showStatus( this.pycommand );
        var self = this;
        var endHandler = function( arg ) { self.processHTML( arg ); };
        this.command = widget.system( this.pycommand, endHandler );
    }
}

function _JMALoader_forceLoading( url ) {
    this.load( url, true );
}


function _JMALoader_cancel() {
    if ( this.command != null ) {
        this.command.cancel();
    }

    this.command   = null;
    if ( this.endHandler != null ) {
        this.endHandler( this );
    }

    this.isRunning = false;
}


 
/// button
function mouseDown( button ) {
    if ( button.state == "enabled" ) {
        button.src = "ButtonImages/" + button.name + "2.png";
    }
}

function mouseUp( button ) {
    if ( button.state == "enabled" ) {
        button.src = "ButtonImages/" + button.name + "1.png";
    }
}

function mouseOut( button ) {
    if ( button.state == "enabled" ) {
        button.src = "ButtonImages/" + button.name + "1.png";
    }
}

function disable( button ) {
    button.src = "ButtonImages/" + button.name + "3.png";
    button.state = "disabled";
}

function enable( button ) {
    button.src = "ButtonImages/" + button.name + "1.png";
    button.state = "enabled";
}


/*
      var flipped = false;
      function flip() {
      if ( flipped ) {
      widget.prepareForTransition("ToFront");
      } else {
      widget.prepareForTransition("ToBack");
      }
      setTimeout( "widget.performTransition();", 0 );
      }

*/
