#!/usr/bin/python
# -*- coding:utf-8 -*- 

import sys, os, os.path, re
import JMAEarthQuake

flag_sqlite = False
try:
    import sqlite
    flag_sqlite = True
except ImportingError: # no SQLite module
    pass

def get_home():
    """Acquire the absolute path of user's home."""
    import tempfile
    file_temp = tempfile.mktemp()
    env = os.system( "/bin/echo $HOME > %s" % file_temp )
    path_home = open( file_temp ).readline().strip()
    os.unlink( file_temp )
    return path_home

filename_db = get_home() + "/Library/Preferences/.db.tiny.widget.earthquake"

if __name__ == "__main__":
    flag_create = False
    maxOutputSize = 4080

    if os.path.exists( filename_db ) == False:
        flag_create = True
        pass

    # access to database and create database if no table exist
    if flag_sqlite:
        cnx = sqlite.connect( filename_db )
        cur= cnx.cursor()
        # check database
        cur.execute( "PRAGMA table_info( earthquake );" )
        rows = cur.fetchall()
        if len( rows ) > 0 and len( rows ) != 3:
            cur.execute( "drop table earthquake;" )
            pass
        elif len( rows ) == 0:
            # create database and indize url
            cur.execute( "create table earthquake ( url NOT NULL, date NOT NULL, text NOT NULL );" )
            cur.execute( "create index urlindex on earthquake ( url );" )
            pass
        pass

    flag_completed = False

    # force load ( load data from internet even if they are in database )
    flag_forced = ( sys.argv.count( "-force" ) > 0 )
    url_request = JMAEarthQuake.JMAEarthQuake.get_base_URL()
    for arg in sys.argv[ 1: ]:
        if arg[ 0: 7 ] == "http://":
            url_request = arg
            break
        pass

    # database management
    if flag_sqlite:
        if flag_forced == False:
            cur.execute( "select url, text from earthquake where url=\"%s\";" % url_request )
            db_out = cur.fetchone()
            if db_out != None and len( db_out ) > 0:
                flag_completed = True
                print db_out[ 1 ] + "<!--END-->"
                pass
            pass
        pass

    # if no entry found, load data from internet
    if flag_completed == False:
        try:
            earthquake = JMAEarthQuake.JMAEarthQuake( url_request )
            m_date = re.search( "quake\\.(\\d+)", url_request )
            if m_date:
                date = m_date.group( 1 )
            else:
                date = url_request
                pass
            text = earthquake.get_string( maxOutputSize )

            # access to database
            if flag_sqlite:
                if flag_forced: 
                    command_sql = 'update earthquake set date="%s", text="%s" where url="%s";' % ( date, text, url_request )
                else:
                    command_sql = 'insert into earthquake ( url, date, text ) values ( "%s", "%s", "%s" );' % ( url_request, date, text )
                    pass
                #print command_sql
                cur.execute( command_sql )
                cnx.commit()
                pass
            print text + "<!--END-->"
            pass
        except JMAEarthQuake.NetowrkExceptionByCurl, e:
            print "NETWORK ERROR"
        except Exception, e:
            raise
            pass
        pass

    # close database
    if flag_sqlite: cnx.close()

    pass
        
